
/* Copyright (c) 2012 by Al Williams

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software is intended to communicate with a GP9 device. Note that the above license covers,
However, this software does not include the firmware that resides inside the GP9. The above license
covers THIS software and not the GP9 firmware, which is not included in this distribution.

More info: http://www.awce.com/gp9.htm
al.williams@awce.com

*/

/** @file
 *  This is the windows-specific gp3 file. Compile and link with
 * gp3lib.c. Do not link with other files (such as gp3cygwin.c). 
 * @author Al Williams al.williams@awce.com
 * @version 1.2
 * @date 4 Feb 2012
*/

#include <QApplication>
#if defined(Q_OS_WIN32)

#include <windows.h>
#include <stdio.h>
#include "gp3linux.h"  // excuse the name

/** If DEBUGOUT=1 then the platform-specific code will generate
 * interesting debug messages.
 * 
 */
#define DEBUGOUT 0
#if DEBUGOUT
#define DEBUG(tag,x) printf("DEBUG: %s=%02X\n",tag,x)
#else
#define DEBUG(tag,x)
#endif

/* This is the part that is platform specific */
/* This is for Windows */

static HANDLE comport;  // file descriptor


/* Open port at 9600 baud */
int gp3openport(const char *port)
{
	DCB dcb;
	COMMTIMEOUTS cto;
    DWORD rv;
    size_t count;
    wchar_t wport[64]=L"\\\\.\\";
    mbstowcs_s(&count,wport+4,sizeof(wport)/sizeof(wchar_t)-4,port,_TRUNCATE);
    comport=CreateFile(wport,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
    if (comport!=INVALID_HANDLE_VALUE)
	{
	   SetupComm(comport,128,128);
	   cto.ReadIntervalTimeout=0;
	   cto.ReadTotalTimeoutMultiplier=100;
	   cto.ReadTotalTimeoutConstant=0;
	   cto.WriteTotalTimeoutMultiplier=0;
	   cto.WriteTotalTimeoutConstant=100;
	   SetCommTimeouts(comport,&cto);
	   dcb.DCBlength=sizeof(dcb);
	   GetCommState(comport,&dcb);
           rv=BuildCommDCB(TEXT("baud=9600 parity=N data=8 stop=1"),&dcb);
	   dcb.fBinary=TRUE;
	   dcb.fDtrControl=DTR_CONTROL_DISABLE;
	   dcb.fOutxDsrFlow=FALSE;
	   dcb.fNull=FALSE;
	   dcb.fRtsControl=RTS_CONTROL_ENABLE;
	   dcb.fOutxCtsFlow=TRUE;
	   dcb.fDsrSensitivity=FALSE;
	   dcb.fOutX=FALSE;
	   dcb.fInX=FALSE;
           rv=SetCommState(comport,&dcb);
	   if (!rv) 
	   {
		   CloseHandle(comport);
           comport=INVALID_HANDLE_VALUE;
	   }
	}
    return comport!=INVALID_HANDLE_VALUE?0:-1;
}

/* Close the port when done */
void gp3closeport()
{
  CloseHandle(comport);
}



/* Write a byte to the GP3 */
int gp3write(int byte)
{
  char c=(char)byte;
  DWORD n=0;
  DEBUG("write",byte);
  WriteFile(comport,&c,1,&n,NULL);
  return n==1;
}

/* Read a byte from the GP3 */
/* User may call this after repeat */
unsigned int gp3read(void)
{
  char c;
  DWORD n;
  unsigned int rv;
  ReadFile(comport,&c,1,&n,NULL);
  rv=((unsigned int)c) & 0xFF;;  
  DEBUG("read",rv);
  return (n==1)?rv:-1;
}



#endif



